from ate_projectdatabase.Types import Types
from ate_projectdatabase.FileOperator import DBObject, FileOperator


class TestTarget:

    @staticmethod
    def add(session: FileOperator, name: str, prog_name: str, hardware: str, base: str, test: str, is_default: bool, is_enabled: bool):
        target = {"name": name, "prog_name": prog_name, "hardware": hardware, "base": base, "test": test,
                  "is_default": is_default, "is_enabled": is_enabled, "is_changed": False, "is_changed": False}
        session.query_with_subtype(Types.Testtarget(), name).add(target)
        session.commit()

    @staticmethod
    def get(session: FileOperator, name: str, hardware: str, base: str, test: str) -> DBObject:
        return session.query(Types.Testtarget())\
                      .filter(lambda TestTarget: (TestTarget.name == name and TestTarget.hardware == hardware and TestTarget.base == base and TestTarget.test == test))\
                      .one()

    @staticmethod
    def get_all(session: FileOperator):
        return session.query(Types.Testtarget())\
                      .all()

    @staticmethod
    def get_tests(session: FileOperator, hardware: str, base: str, test_target: str) -> list:
        return session.query(Types.Testtarget())\
                      .filter(lambda TestTarget: (TestTarget.name == test_target and TestTarget.hardware == hardware and TestTarget.base == base))\
                      .all()

    @staticmethod
    def get_for_hardware_base_test(session: FileOperator, hardware: str, base: str, test: str) -> list:
        return session.query(Types.Testtarget())\
                      .filter(lambda TestTarget: (TestTarget.hardware == hardware and TestTarget.base == base and TestTarget.test == test))\
                      .all()

    @staticmethod
    def get_for_program(session: FileOperator, prog_name: str) -> list:
        return session.query(Types.Testtarget())\
                      .filter(lambda TestTarget: TestTarget.prog_name == prog_name)\
                      .all()

    @staticmethod
    def get_for_test(session: FileOperator, test_name: str, hardware: str, base: str) -> list:
        return session.query(Types.Testtarget())\
                      .filter(lambda TestTarget: TestTarget.test == test_name and TestTarget.hardware == hardware and TestTarget.base == base)\
                      .all()

    @staticmethod
    def exists(session: FileOperator, name: str, hardware: str, base: str, test: str, prog_name: str) -> bool:
        num_items = session.query(Types.Testtarget())\
                           .filter(lambda TestTarget: (TestTarget.name == name and TestTarget.prog_name == prog_name and TestTarget.hardware == hardware and TestTarget.base == base and TestTarget.test == test))\
                           .count()
        return num_items != 0

    @staticmethod
    def remove(session: FileOperator, name: str, test: str, hardware: str, base: str):
        session.query(Types.Testtarget())\
               .filter(lambda TestTarget: (TestTarget.name == name and TestTarget.test == test and TestTarget.hardware == hardware and TestTarget.base == base))\
               .delete()
        session.commit()

    @staticmethod
    def remove_for_test_program(session: FileOperator, prog_name: str):
        session.query(Types.Testtarget())\
               .filter(lambda TestTarget: TestTarget.prog_name == prog_name)\
               .delete()
        session.commit()

    @staticmethod
    def get_changed_test_targets(session: FileOperator, hardware: str, base: str, prog_name: str) -> list:
        return session.query(Types.Testtarget())\
                      .filter(lambda TestTarget: (TestTarget.is_changed and TestTarget.prog_name == prog_name and TestTarget.hardware == hardware and TestTarget.base == base))\
                      .all()

    @staticmethod
    def update_changed_state_test_targets(session: FileOperator, hardware: str, base: str, prog_name: str) -> list:
        tests = session.query(Types.Testtarget())\
                       .filter(lambda TestTarget: (TestTarget.prog_name == prog_name and TestTarget.hardware == hardware and TestTarget.base == base))\
                       .all()
        for test in tests:
            test.is_changed = False

        session.commit()

    @staticmethod
    def set_default_state(session: FileOperator, name: str, hardware: str, base: str, test: str, is_default: bool):
        test_target = TestTarget.get(session, name, hardware, base, test)
        test_target.is_default = is_default
        session.commit()

    @staticmethod
    def toggle(session: FileOperator, name: str, hardware: str, base: str, test: str, enable: bool):
        test_target = TestTarget.get(session, name, hardware, base, test)
        test_target.is_enabled = enable
        session.commit()

    @staticmethod
    def update_program_name(session: FileOperator, prog_name: str, new_prog_name: str):
        test_targets = TestTarget.get_for_program(session, prog_name)
        for test_target in test_targets:
            test_target.prog_name = new_prog_name
            session.commit()

    @staticmethod
    def update_test_changed_flag(session: FileOperator, name: str, hardware: str, base, test: str, is_changed: bool):
        test_target = TestTarget.get(session, name, hardware, base, test)
        test_target.is_changed = is_changed
        session.commit()



from ate_projectdatabase.Types import Types
from ate_projectdatabase.FileOperator import DBObject, FileOperator


class Device:

    @staticmethod
    def get(session: FileOperator, name: str) -> DBObject:
        return session.query(Types.Device())\
                      .filter(lambda device: device.name == name)\
                      .one()

    @staticmethod
    def get_all(session: FileOperator) -> list:
        return session.query(Types.Device())\
                      .all()

    @staticmethod
    def add(session: FileOperator, name: str, hardware: str, package: str, definition: dict, is_enabled: bool):
        # ToDo: implement constraints, i.e. hardware should exist
        device = {"name": name, "hardware": hardware, "package": package, "definition": definition, "is_enabled": is_enabled}
        session.query_with_subtype(Types.Device(), name).add(device)
        session.commit()

    @staticmethod
    def update(session: FileOperator, name: str, hardware: str, package: str, definition: dict):
        device = Device.get(session, name)
        device.hardware = hardware
        device.package = package
        device.definition = definition
        session.commit()

    @staticmethod
    def update_state(session: FileOperator, name: str, is_enabled: bool):
        device = Device.get(session, name)
        device.is_enabled = is_enabled
        session.commit()

    @staticmethod
    def remove(session: FileOperator, name: str):
        session.query(Types.Device())\
               .filter(lambda Device: Device.name == name)\
               .delete()
        session.commit()

    @staticmethod
    def get_definition(session: FileOperator, name: str) -> dict:
        device = Device.get(session, name)
        return device.definition

    @staticmethod
    def get_all_for_hardware(session: FileOperator, hardware: str) -> list:
        return session.query(Types.Device())\
                      .filter(lambda Device: Device.hardware == hardware)\
                      .all()



from ate_projectdatabase.Utils import DB_KEYS
from ate_projectdatabase.FileOperator import DBObject, FileOperator
from ate_projectdatabase.Types import Types

from ate_projectdatabase.Hardware.ParallelismStore import ParallelismStore


class Hardware:

    @staticmethod
    def get(session: FileOperator, name: str) -> DBObject:
        return session.query(Types.Hardware())\
                      .filter(lambda Hardware: Hardware.name == name)\
                      .one()

    @staticmethod
    def get_all(session: FileOperator) -> list:
        return session.query(Types.Hardware()).all()

    @staticmethod
    def add(session: FileOperator, name: str, definition: dict, is_enabled: bool):
        hw = {
            DB_KEYS.HARDWARE.NAME: name,
            DB_KEYS.HARDWARE.DEFINITION.KEY(): definition,
            DB_KEYS.HARDWARE.IS_ENABLED: is_enabled
        }
        session.query_with_subtype(Types.Hardware(), name).add(hw)
        session.commit()

    @staticmethod
    def remove(session: FileOperator, name: str):
        session.query(Types.Hardware())\
               .filter(lambda Hardware: Hardware.name == name)\
               .delete()
        session.commit()

    @staticmethod
    def update_state(session: FileOperator, name: str, is_enabled: bool):
        hw = Hardware.get(session, name)
        hw.is_enabled = is_enabled
        session.commit()

    @staticmethod
    def get_state(session: FileOperator, name: str):
        return Hardware.get(session, name).is_enabled

    @staticmethod
    def update_definition(session: FileOperator, name: str, definition: dict):
        hw = Hardware.get(session, name)
        hw.definition = definition
        session.commit()

    @staticmethod
    def get_definition(session: FileOperator, name: str) -> dict:
        hardware = Hardware.get(session, name)
        return hardware.definition

    @staticmethod
    def update_parallelism_store(session: FileOperator, hw_name: str, parallelism_store: ParallelismStore):
        hw_definition = Hardware.get_definition(session, hw_name)
        hw_definition[DB_KEYS.HARDWARE.DEFINITION.PARALLELISM.KEY()] = parallelism_store.serialize()
        session.commit()

    @staticmethod
    def get_parallelism_store(session: FileOperator, hw_name: str) -> ParallelismStore:
        hw_definition = Hardware.get_definition(session, hw_name)
        return ParallelismStore.from_database(hw_definition[DB_KEYS.HARDWARE.DEFINITION.PARALLELISM.KEY()])



from ate_projectdatabase.Types import Types
from ate_projectdatabase.FileOperator import DBObject, FileOperator


class Die:

    @staticmethod
    def get(session: FileOperator, name: str) -> DBObject:
        return session.query(Types.Die()).filter(lambda Die: Die.name == name).one()

    @staticmethod
    def get_all(session: FileOperator) -> list:
        return session.query(Types.Die()).all()

    @staticmethod
    def add(session: FileOperator, name, hardware, maskset, quality, grade, grade_reference, type, customer, is_enabled):
        die = {"name": name, "hardware": hardware, "maskset": maskset, "quality": quality, "grade": grade,
               "grade_reference": grade_reference, "type": type, "customer": customer, "is_enabled": is_enabled}
        session.query_with_subtype(Types.Die(), name).add(die)
        session.commit()

    @staticmethod
    def update(session: FileOperator, name: str, hardware: str, maskset: str, quality: str, grade: str, grade_reference: str,
               type: str, customer: str):
        die = Die.get(session, name)
        die.hardware = hardware
        die.maskset = maskset
        die.quality = quality
        die.grade = grade
        die.grade_reference = grade_reference
        die.type = type
        die.customer = customer
        session.commit()

    @staticmethod
    def update_state(session: FileOperator, name: str, is_enabled: bool):
        die = Die.get(session, name)
        die.is_enabled = is_enabled
        session.commit()

    @staticmethod
    def remove(session: FileOperator, name: str):
        session.query(Types.Die())\
               .filter(lambda Die: Die.name == name)\
               .delete()
        session.commit()

    @staticmethod
    def get_hardware(session: FileOperator, name: str) -> list:
        return session.query(Types.Die())\
                      .filter(lambda Die: Die.hardware == name and Die.is_enabled)\
                      .all()

    @staticmethod
    def get_die(session: FileOperator, name: str) -> DBObject:
        return session.query(Types.Die())\
                      .filter(lambda Die: Die.name == name)\
                      .one()

    @staticmethod
    def get_all_for_hardware(session: FileOperator, hardware: str) -> list:
        return session.query(Types.Die())\
                      .filter(lambda Die: Die.hardware == hardware)\
                      .all()

    @staticmethod
    def get_all_for_maskset(session: FileOperator, maskset: str) -> list:
        return session.query(Types.Die())\
                      .filter(lambda Die: Die.maskset == maskset)\
                      .all()



from ate_projectdatabase.Types import Types
from ate_projectdatabase.FileOperator import DBObject, FileOperator


class QualificationFlowDatum:

    # To Do: Check how this is actually used.
    # def get_definition(self):
    #     if self.data is None:
    #         return {'product': self.product}

    #     flow_data = pickle.loads(self.data)
    #     if 'product' not in flow_data:
    #         flow_data['product'] = self.product
    #     if 'name' not in flow_data:
    #         flow_data['name'] = self.name
    #     if 'type' not in flow_data:
    #         flow_data['type'] = self.type
    #     return flow_data

    @staticmethod
    def get_data_for_flow(session: FileOperator, flow_type: str, product: str) -> DBObject:
        return session.query(Types.Qualification())\
                      .filter(lambda QualificationFlowDatum: (QualificationFlowDatum.type == flow_type and QualificationFlowDatum.product == product))\
                      .all()

    @staticmethod
    def _get_by_name(session: FileOperator, name: str) -> DBObject:
        return session.query(Types.Qualification())\
                      .filter(lambda QualificationFlowDatum: QualificationFlowDatum.name == name)\
                      .one_or_none()

    @staticmethod
    def add_or_update_qualification_flow_data(session: FileOperator, quali_flow_data: DBObject):
        item = QualificationFlowDatum._get_by_name(session, quali_flow_data.read_attribute("name"))
        if item is not None:
            session.filter(lambda x: x.name == item.name).delete()

        session.add(quali_flow_data.to_dict())
        session.commit()

    @staticmethod
    def remove(session: FileOperator, quali_flow_data: DBObject):
        session.query(Types.Qualification()) \
               .filter(lambda QualificationFlowDatum: (QualificationFlowDatum.name == quali_flow_data.name and QualificationFlowDatum.type == quali_flow_data.type and QualificationFlowDatum.product == quali_flow_data.product))\
               .delete()
        session.commit()



from ate_projectdatabase.Types import Types
from ate_projectdatabase.FileOperator import DBObject, FileOperator


class Package:
    @staticmethod
    def add(session: FileOperator, name: str, leads: int, is_naked_die: bool, is_enabled: bool):
        package = {"name": name, "leads": leads, "is_naked_die": is_naked_die, "is_enabled": is_enabled}
        session.query_with_subtype(Types.Package(), name).add(package)
        session.commit()

    @staticmethod
    def update(session: FileOperator, name: str, leads: int, is_naked_die: bool, is_enabled=True):
        package = session.query(Types.Package())\
                         .filter(lambda package: package.name == name)\
                         .one()
        package.leads = leads
        package.is_naked_die = is_naked_die
        package.is_enabled = is_enabled
        session.commit()

    @staticmethod
    def update_state(session: FileOperator, name: str, state: bool):
        package = session.query(Types.Package())\
                         .filter(lambda package: package.name == name)\
                         .one()
        package.is_enabled = state
        session.commit()

    @staticmethod
    def get(session: FileOperator, name: str) -> DBObject:
        return session.query(Types.Package())\
                      .filter(lambda package: package.name == name)\
                      .one_or_none()

    @staticmethod
    def get_all(session: FileOperator) -> list:
        return session.query(Types.Package()).all()

    @staticmethod
    def remove(session: FileOperator, name: str):
        session.query(Types.Package())\
               .filter(lambda package: package.name == name)\
               .delete()
        session.commit()



from ate_projectdatabase.Types import Types
from ate_projectdatabase.FileOperator import DBObject, FileOperator


class Settings:

    @staticmethod
    def _generate_settings(session: FileOperator):
        settings = {
            "quality_grade": ""
        }
        session.query(Types.Settings()).add(settings)
        session.commit()

    @staticmethod
    def get(session: FileOperator) -> DBObject:
        if session.query(Types.Settings()).one_or_none() is None:
            Settings._generate_settings(session)
        return session.query(Types.Settings()).one()

    @staticmethod
    def set_quality_grade(session: FileOperator, quality_grade: str):
        settings = Settings.get(session)
        settings.quality_grade = quality_grade
        session.commit()

    @staticmethod
    def get_quality_grade(session: FileOperator) -> str:
        setting = Settings.get(session)
        return setting.quality_grade



from ate_projectdatabase.Types import Types
from ate_projectdatabase.FileOperator import DBObject, FileOperator


class Product:

    @staticmethod
    def get(session: FileOperator, name: str) -> DBObject:
        return session.query(Types.Product())\
                      .filter(lambda Product: Product.name == name)\
                      .one_or_none()

    @staticmethod
    def get_all(session: FileOperator) -> list:
        return session.query(Types.Product()).all()

    @staticmethod
    def add(session: FileOperator, name: str, device: str, hardware: str, quality: str, grade: str,
            grade_reference: str, type: str, customer: str, is_enabled=True):
        # existing_products = Product.get_all(session)
        # if name in existing_products:
        #     raise KeyError(f"product '{name}' already exists")
        product = {"name": name, "device": device, "hardware": hardware, "is_enabled": is_enabled,
                   "grade": grade, "quality": quality, "grade_reference": grade_reference, "type": type,
                   "customer": customer}
        session.query_with_subtype(Types.Product(), name)
        session.add(product)
        session.commit()

    @staticmethod
    def update(session: FileOperator, name: str, device: str, hardware: str,
               quality: str, grade: str, grade_reference: str, type: str,
               customer: str, is_enabled=True):
        prod = Product.get(session, name)
        prod.device = device
        prod.hardware = hardware
        prod.quality = quality
        prod.grade = grade
        prod.grade_reference = grade_reference
        prod.type = type
        prod.customer = customer
        prod.is_enabled = is_enabled
        session.commit()

    @staticmethod
    def update_state(session: FileOperator, name: str, state: bool):
        product = session.query(Types.Product())\
                         .filter(lambda Product: Product.name == name)\
                         .one()
        product.is_enabled = state
        session.commit()

    @staticmethod
    def get_for_hardware(session: FileOperator, hardware: str) -> list:
        return session.query(Types.Product())\
                      .filter(lambda Product: Product.hardware == hardware)\
                      .all()

    @staticmethod
    def get_for_device(session: FileOperator, device_name: str) -> list:
        return session.query(Types.Product())\
                      .filter(lambda Product: Product.device == device_name)\
                      .all()

    @staticmethod
    def remove(session: FileOperator, name: str):
        session.query(Types.Product())\
               .filter(lambda Product: Product.name == name)\
               .delete()
        session.commit()

    @staticmethod
    def get_all_for_hardware(session: FileOperator, hardware: str) -> list:
        return session.query(Types.Product())\
                      .filter(lambda Product: Product.hardware == hardware)\
                      .all()



from enum import Enum


class Types(Enum):
    Maskset = 'masksets'
    Package = 'package'
    Device = 'device'
    Die = 'die'
    Hardware = 'hardware'
    Product = 'product'
    Program = 'program'
    Qualification = 'qualification'
    Sequence = 'sequence'
    Test = 'test'
    Testtarget = 'testtarget'
    Group = 'group'
    Settings = 'settings'
    Version = 'version'

    def __call__(self):
        return self.value
